﻿//DESCRIPTION:Create a hyperlink to the URL on the clipboard for the selected text or object

// Modified 2023-08-08
// Keith Gilbert, Gilbert Consulting
// www.gilbertconsulting.com

Pre();
  
function Pre () {  
	if (app.documents.length < 1) {
		alert("No InDesign documents are open. Please open a document and try again.");
		return;
	}
	if (locateCharacterStyle("Hyperlink") == "not found") {
		alert("This script requires a character style named 'Hyperlink'. Please create a character style named 'Hyperlink' and try again.");
		return;
	}
	if(app.selection.length != 1) {
		alert("Please select some text or a single object to which you want to add a hyperlink and try again.");
		return;
	}
	switch(app.selection[0].constructor.name){
		case "Character":
		case "Word":
		case "TextStyleRange":
		case "Line":
		case "Paragraph":
		case "TextColumn":
		case "Text":
			// Main("text");            
			app.doScript('Main("text");', undefined, ["text"], UndoModes.ENTIRE_SCRIPT,"Run Script"); // This makes the script un-doable
			break;
        case "TextFrame":
		case "TextBox":
		case "Rectangle":
		case "Oval":
		case "Polygon":
		case "GraphicLine":
		case "Image":
		case "PDF":
		case "EPS":
		case "Group":
			// Main("object");
			app.doScript('Main("object");', undefined, ["object"], UndoModes.ENTIRE_SCRIPT,"Run Script"); // This makes the script un-doable
			break;
		case "Table":
		case "Cell":
		default:
			alert("Please select some text or a single object and try again.");
			break;
	}  
}  

function Main(mySelectionType) {
	var myDoc = app.activeDocument;
	var mySelection = app.selection[0];
    // Create a temporary layer and text frame to retrieve the clipboard contents
    var myTempLayer = myDoc.layers.add();
    var myTempTextFrame = myDoc.textFrames.add(myTempLayer);
    myTempTextFrame.insertionPoints[-1].select();
    app.paste();
    var myURL = myTempTextFrame.parentStory.contents;
    myTempTextFrame.remove();
    myTempLayer.remove();
	// If the selection already contains a hyperlink, remove it
	myRemoveHyperlink(myDoc, mySelection, mySelectionType);
    // Create the hyperlink
	if (mySelectionType == "text") {
		var mySource = myDoc.hyperlinkTextSources.add(mySelection);
		var myDestination = myDoc.hyperlinkURLDestinations.add(myURL, {hidden:true});
		var myHyperLink = myDoc.hyperlinks.add(mySource,myDestination);
		try {
			myHyperLink.name = mySelection.contents;
		}
		catch (myError) {
			var n=1;
			while (myHyperLink.name == "Hyperlink") {
				try {
					myHyperLink.name = mySelection.contents+" "+String(n);
				}
				catch (myError2) {
					n++;	
				}
			}
		}
		mySelection.appliedCharacterStyle = myDoc.characterStyles.itemByID(locateCharacterStyle("Hyperlink"));
	}
	else {
		var mySource = myDoc.hyperlinkPageItemSources.add(mySelection);
		var myDestination = myDoc.hyperlinkURLDestinations.add(myURL, {hidden:true});
		var myHyperLink = myDoc.hyperlinks.add(mySource,myDestination);
		try {
			myHyperLink.name = "<"+mySelection.constructor.name.toLowerCase()+">";
		}
		catch (myError) {
			var n=1;
			while (myHyperLink.name == "Hyperlink") {
				try {
					myHyperLink.name = "<"+mySelection.constructor.name.toLowerCase()+">"+" "+String(n);
				}
				catch (myError2) {
					n++;	
				}
			}
		}
	}
}
// Find a character style possibly contained inside a style group. Return id of the character style.
function locateCharacterStyle(styleNameToLocate, pathSeparator) {
	var myDoc = app.activeDocument;
    var rootPath = "";
    if (pathSeparator !== undefined) {
        rootPath = pathSeparator;
    }
	var retVal = locateCharacterStyleRecursive(myDoc, styleNameToLocate, pathSeparator, rootPath);
	if (! retVal) {
		return "not found";
	}
	return retVal.id;
}
// Find character styles that are in style groups. Returns the character style or null if not found.
function locateCharacterStyleRecursive(characterStyleContainer, styleNameToLocate, pathSeparator, parentPath) {
	var retVal = null;
	do { 
		try {
			var characterStyles = characterStyleContainer.characterStyles;
			var numCharacterStyles = characterStyles.length; 
			for (
				var characterStyleIdx = 0; 
				retVal == null && characterStyleIdx < numCharacterStyles;
				characterStyleIdx++) {
				var characterStyle = characterStyles.item(characterStyleIdx);
				if (characterStyle.isValid) {
					var characterStyleName = characterStyle.name;
                      if (pathSeparator) {
                          characterStyleName = parentPath + characterStyleName;
                      }
					if (characterStyleName == styleNameToLocate) {
						retVal = characterStyle;
					}
				}
			}
			if (retVal) {
				break;
			}
			var characterStyleGroups = characterStyleContainer.characterStyleGroups;
			var numCharacterStyleGroups = characterStyleGroups.length;
			for (
				var characterStyleGroupIdx = 0; 
				retVal == null && characterStyleGroupIdx < numCharacterStyleGroups;
				characterStyleGroupIdx++) {

				var characterStyleGroup = characterStyleGroups.item(characterStyleGroupIdx);
				if (characterStyleGroup.isValid) {
					var path = parentPath;
                      if (pathSeparator) {
                          var characterStyleGroupName = characterStyleGroup.name;
                          path += characterStyleGroupName + pathSeparator;
                      }
					retVal = locateCharacterStyleRecursive(characterStyleGroup, styleNameToLocate, pathSeparator, path);
				}
			}
		}
		catch (err) {
		}
	}
	while (false);
	return retVal;
}
// Remove the hyperlink whose source is the specified text or object
function myRemoveHyperlink(myDoc,mySelection,mySelectionType) {
	if (mySelectionType == "text") {
		var myHyperLinkArray = mySelection.findHyperlinks();
		for (var i = myHyperLinkArray.length-1; i >= 0; i--) {
			var myHyperLink = myHyperLinkArray[i];
			myHyperLink.appliedCharacterStyle = myDoc.characterStyles.item("[None]");
			myHyperLink.remove();
		}
		return;
	}
	else { // Selection is an object
		for (var i = myDoc.hyperlinks.length-1; i >= 0; i--) {
			var myHyperLink = myDoc.hyperlinks[i];
			try { // Hyperlink source is an object
				if (myDoc.hyperlinks[i].source.sourcePageItem == mySelection) {
					myDoc.hyperlinks[i].source.remove();
					if (myDoc.hyperlinks[i].destination.hidden == true) { // Hyperlink destination is not a shared destination
						myDoc.hyperlinks[i].destination.remove();
					}
					else {
						myDoc.hyperlinks[i].remove();
					}
					return;
				}	
			}
			catch (e) {} // Hyperlink source is text
		}
	}
}